/*
*
* Gtk+2.x  Utility routines: About and license dialog Window
* Copyright (C) 2002-2006 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/

/********************* About and license file viewer Widget *******************************/

/* Needs TextCommon.c */
#include <gtk/gtk.h>
#include <assert.h>
#include <stdlib.h>

static int about_lock = 0;
static GtkWidget *about_window;
static GtkWidget *about_text;

void about_wrap_text(void)
{
	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(about_text),
								(gtk_text_view_get_wrap_mode (GTK_TEXT_VIEW(about_text)) != GTK_WRAP_NONE) ?
								GTK_WRAP_NONE :
								GTK_WRAP_WORD);
}

static GtkActionEntry about_entries[] = {
	/* name,		stock id,	label,				accelerator,		tooltip,	callback */
	{"OptionsMenu", NULL,		MNU_STR_OPTIONS,		NULL,				NULL,		NULL},	
};

static GtkToggleActionEntry about_check_entries[] = 
{
	{"Wrap", NULL,								/* name, stock id */
	MNU_STR_WRAP, SHC_OPTIONS_WRAP,				/* label, accelerator */     
	NULL,										/* tooltip */
	G_CALLBACK (about_wrap_text),				/* callback */ 
	FALSE },									/* is_active */
};

static const gchar *about_ui_info = 
"<ui>"
"  <menubar name='MenuBar'>"
"    <menu action='OptionsMenu'>"
"      <menuitem action='Wrap'/>"
"    </menu>"
"  </menubar>"
"</ui>";

static struct MenuEntry about_menu_entries[] = {
{ about_entries,	NULL,					NULL,	G_N_ELEMENTS (about_entries),			0, NULL},
{ NULL,				about_check_entries,	NULL,	G_N_ELEMENTS (about_check_entries),		0, NULL},
{ NULL,				NULL,					NULL,	0,										0, NULL},
/* Set last to NULL */ 
};

static void license_callback(GtkWidget *widget, int *ptr_status)
{
	DBG_FUNCTION_NAME;
	assert(widget != NULL);
	
	SET2(*ptr_status,(*ptr_status == 0) ? 1 : 0);
}

static void about_quit(void)
{
	if (about_lock) return;

	gtk_main_quit();
	gtk_widget_destroy(about_window);
}

int about_window_create(char * title, int x, int y, int border, char *labeltext, FILE *fd, int first_time)
{
	GtkWidget *menubar;
	GtkWidget *table;
	GtkWidget *frame;
	GtkWidget *label;
	GtkWidget *bbox;
	GtkWidget *check_button;
	GtkWidget *button;
	int *ptr_status;
	int status = 0;

	DBG_FUNCTION_NAME;

	ptr_status = &status;
	
	/* Make the gui first  */
	about_window = window_create(x, y, border, title, about_quit, NULL, MODAL);
	table = table_create( 20, 18, about_window , 0, NOT_HOMOGENEUS);
	menubar = main_menu_create(GTK_WIDGET (about_window), about_menu_entries, about_ui_info, NULL);
	gtk_table_attach_defaults (GTK_TABLE(table),GTK_WIDGET(menubar), 0, 20, 0, 1);
	frame = frame_create_on_table( NULL, table, 0, 20, 1, 4);
	label = label_create(labeltext);	
	gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
	gtk_container_add(GTK_CONTAINER(frame), label);	
	about_text = textpad_create_on_table(NOT_EDITABLE, NULL , NULL, NO_WRAP, SCROLL, table,  0, 20, 4, 16);
	bbox = default_button_box_create_on_table(HORIZONTAL, table, 7, 20, 16, 18);
	button = button_create_in_container(STR_OK, about_quit, NULL, bbox);
	
	if (first_time == FIRST_TIME) {
		check_button = check_create_on_table(STR_I_AGREE_DONT_SHOW_IT_AGAIN, G_CALLBACK(license_callback),
										ptr_status, INACTIVE, table, 0, 3, 17, 18);
	}

	gtk_widget_show_all(about_window);
	about_lock = 1;	
	/* Read from fd */
	if (fd) {
		read_from_stream_and_display_text(about_text, fd, 0, NULL);
	} else {
		dialog_window_and_beep(STR_ERROR, ONE_BUTTON, NULL, 1, STR_CANNOT_OPEN_FILE);
	}
	about_lock = 0;
	main_loop(&about_window);
	return status;
}

/*************** End of  About and license file viewer Widget ********************************/
