/* 1.52 */
/* vi: set sw=4 ts=4: */
/*
 * Mini ps implementation(s) for busybox,
 * hacked for Mini Gtk+ File Manager by Tito Ragusa
 *
 * Copyright (C) 1999-2004 by Erik Andersen <andersen@codepoet.org>
 * Copyright (C) 2004-2006 by Tito Ragusa <farmatito@tiscali.it>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <gtk/gtk.h>
#include <assert.h> 
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <pwd.h>

GtkWidget *pslist;
char *pslist_selection = NULL;

void pslist_f(GtkWidget *tree)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	GtkTreeSelection *selection;

	DBG_FUNCTION_NAME;
	assert(tree!=NULL);
	
	if(GTK_IS_TREE_VIEW(tree))
		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (tree));
	else
		selection = GTK_TREE_SELECTION(tree);

	if (gtk_tree_selection_get_selected (selection, &model, &iter)) {
		gtk_tree_model_get (model, &iter, 0 /* pid column */ , &pslist_selection, -1);
		debug_err_msg("%s(): you selected : %s",__FUNCTION__, pslist_selection);
	}
}

void ps_main(GtkWidget *ps_list)
{
	procps_status_t * p = NULL;
	struct passwd *pw;
	char *row[5];

	DBG_FUNCTION_NAME;
	assert(pslist != NULL);
	
	while ((p = procps_scan(p, PSSCAN_DEF)) != NULL) {
		row[0] = xmake_message("%u", p->pid);
		row[1] = (pw = getpwuid(p->uid)) ? xstrdup(pw->pw_name) : xmake_message("%u", p->uid);
		row[2] = (p->rss != 0) ? (char *)make_human_readable_str(p->rss, 1024, 0) : "";
		row[3] = (p->state != 0 && p->state[0] != 0) ? p->state : "";
		row[4] = (p->cmd != 0 && p->cmd[0] != 0) ? p->cmd : xmake_message("[%s]",p->comm);
		add_data_to_list(ps_list,(char **) row, 5, NO_AUTOSCROLL);
		xfree(&row[0]);
		xfree(&row[1]);
		if (!(p->cmd != 0 && p->cmd[0] != 0)) xfree(&row[4]);
	}
}

void refresh_ps(GtkWidget *widget, GtkWidget *ps_list)
{
	DBG_FUNCTION_NAME;
	
	assert(widget != NULL);
	clear_list(ps_list);
	ps_main(ps_list);
	pslist_selection = NULL;
}

void kill_pid(GtkWidget *widget)
{
	pid_t pid;

	DBG_FUNCTION_NAME;
	assert(widget != NULL);
			
	if (pslist_selection == NULL) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_NO_PROCESS_SELECTED);
		return;
	}

	if (dialog_window(STR_WARNING, TWO_BUTTONS, NULL, 2, STR_REALLY_KILL_PID_S,  pslist_selection) != 0) {
		pslist_selection = NULL;
		return;
	}

	pid = strtol(pslist_selection, NULL, 0);
	if (kill(pid, 9) != 0) {
		dialog_window(STR_ERROR, ONE_BUTTON, NULL, 1, STR_COULD_NOT_KILL_S_S, 
					pslist_selection, strerror(errno));
	}
	refresh_ps(widget, pslist);
}

void ps_window(void)
{
	GtkWidget *p_window;
	GtkWidget *table;
	GtkWidget *bbox;
	GtkWidget *button;
		
	p_window = window_create(600, 400, 2, STR_PROCESS_MANAGER, GTK_SIGNAL_FUNC(gtk_main_quit), NULL,  MODAL);
	table = table_create(5, 10,  p_window , 2, HOMOGENEUS);
	pslist = string_list_create_on_table(5, GTK_SIGNAL_FUNC(pslist_f), table, 0, 5, 0, 8, SHOW, NOT_EDITABLE,
				" "STR_PID, 1.0, " "STR_UID, 0.0, " "STR_RSS, 1.0," "STR_STATUS, 0.0, " "STR_COMMAND, 0.0);
	bbox = default_button_box_create_on_table( HORIZONTAL, table, 0, 5, 8, 10);
	button = button_create_in_container(STR_KILL, GTK_SIGNAL_FUNC(kill_pid) , pslist, bbox);
	button = button_create_in_container(STR_REFRESH, GTK_SIGNAL_FUNC(refresh_ps) , pslist, bbox);	
	button = button_create_in_container(STR_QUIT, GTK_SIGNAL_FUNC(window_close) , p_window, bbox);
	/* Don't remove this */
	gtk_widget_show_all(p_window);
	ps_main(pslist);
	pslist_selection = NULL;
	main_loop(&p_window);
	return;
}

