/*
*
* Gtk+1.2-Gtk+2.2 Utility routines
* Copyright (C) 2002-2003 by Tito Ragusa <tito-wolit@tiscali.it>
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License as
* published by the Free Software Foundation; either version 2 of
* the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
* Please read the COPYING and README file!!!
*
* Report bugs to <tito-wolit@tiscali.it>
*
*/
#ifndef	__HAVE_MYGTK_
#define	__HAVE_MYGTK_ 1

#include <gtk/gtk.h>
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>

/* This is to avoid a compiler warning for `index` with -Wshadow and Gtk-+-1.2 libs */
#ifndef USE_GTK_2_0
	#ifdef IS_LINUX
		#undef  __USE_BSD
		#include <string.h>
		#define  __USE_BSD
	#endif
#else
	#include <string.h>
#endif

/* For Gtk+1.2.x and Gtk+2.x.x compatibility */
#ifdef USE_GTK_2_0
	#define signal_connect					g_signal_connect
	#define set_editable(x,y)				gtk_editable_set_editable(GTK_EDITABLE (x),y)
	#define get_radio_button_group(x)		gtk_radio_button_get_group(GTK_RADIO_BUTTON (x))
	#define	set_position					gtk_window_set_position
	#define clear_list(x)					gtk_list_store_clear (GTK_LIST_STORE(gtk_tree_view_get_model (GTK_TREE_VIEW(x))))
	#define signal_handlers_destroy			g_signal_handlers_destroy
#else
	#define signal_connect					gtk_signal_connect
	#define set_editable(x,y)				gtk_entry_set_editable (GTK_ENTRY (x), y)
	#define get_radio_button_group(x)		gtk_radio_button_group (GTK_RADIO_BUTTON (x))
	#define	set_position					gtk_window_position
	#define clear_list(x)					gtk_clist_clear(GTK_CLIST (x))
	#define signal_handlers_destroy			gtk_signal_handlers_destroy
#endif
#ifdef USE_GTK_2_4
	#define timeout_remove					g_source_remove
	#define timeout_add						g_timeout_add
#else
	#define timeout_remove					gtk_timeout_remove
	#define timeout_add						gtk_timeout_add
#endif

#define DEF_BUFFER_SIZE		 512
#define NOT_EDITABLE	((int) 0)
#define EDITABLE		((int) 1)
#define ACTIVE 			((int) 0)
#define INACTIVE		((int) 1)
#define HORIZONTAL		((int) 0)
#define VERTICAL		((int) 1)
#define	NOT_MODAL		((int) 0)
#define	MODAL			((int) 1)
#define SHOW			((int) 0)
#define HIDE			((int) 1)
#define WRAP			((int) 0)
#define NO_WRAP			((int) 1)
#define SCROLL			((int) 0)
#define NO_SCROLL		((int) 1)

#define TEXT_NOT_MODIFIED	((int) 0)
#define TEXT_MODIFIED		((int) 1)

#define	HIDE_FILE_OP_BUTTONS	((int) 0)
#define	SHOW_FILE_OP_BUTTONS	((int) 1)

#define	ONE_BUTTON				((int) 1)
#define	ONE_BUTTON_ONE_CHECK	((int) 2)
#define	TWO_BUTTONS				((int) 3)
#define	TWO_BUTTONS_ONE_CHECK	((int) 4)
#define	THREE_BUTTONS			((int) 5)

#define	NOT_HOMOGENEUS	((int) 0)
#define HOMOGENEUS		((int) 1)
#define IGNORED			((int) -1)

#define	NOT_FIRST_TIME	((int) 0)
#define FIRST_TIME		((int) 1)

#ifdef DEBUG
#define RESET(x)		err_msg(#x": %d",(x)=0)
#define SET(x) 			err_msg(#x": %d",(x)=1)
#define SET2(x,y) 		err_msg(#x": %d",(x)=(y))
#else
#define RESET(x) 		(x)=0
#define SET(x) 			(x)=1
#define SET2(x,y)		(x)=(y)
#endif

#define IS_SET(x)		(x)==1
#define IS_NOT_SET(x)	(x)==0

#define ACTIVITY TRUE
#define PULSE    FALSE
#define USE_VALUE		((int) 1)
#define NOT_USE_VALUE	((int) 0)

#define SCREEN_MARGIN 100

void window_close(GtkWidget *widget, GtkWidget *window);
GtkWidget *main_loop(GtkWidget *window);
void attach_to_table(GtkWidget *table, GtkWidget *widget, int start_col, int end_col, int start_row, int end_row);

/********************* Text box widget ***************************************/

GtkWidget *text_box_create_on_table(char *text, int editable, GtkWidget *table,
								 int start_col, int end_col, int start_row, int end_row);

/********************* End of Text box widget ********************************/

/********************* Standard radio button widget **************************/

GtkWidget *radio_create(GtkWidget *last_of_group, char *label,
			 GtkSignalFunc func, gpointer func_data, int active );

/****************End of Standard radio button widget *************************/

/********************* Radio button on table widget **************************/

GtkWidget *radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func, gpointer func_data,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row );
GtkWidget *default_radio_create_on_table(GtkWidget *last_of_group, char *label, GtkSignalFunc func,
										int active, GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row );

/****************End of radio button on table widget *************************/

/********************* Check button widgets **************************/

GtkWidget *check_create(char *label, GtkSignalFunc func, gpointer func_data, int active );
GtkWidget *check_create_on_table(char *label, GtkSignalFunc func, gpointer func_data, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );
GtkWidget *default_check_create_on_table(char *label, GtkSignalFunc func, int active,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );

/***************End of Check button widgets **********************************/

/********************* Standard button widget ********************************/

GtkWidget *button_create(char *label, GtkSignalFunc func, gpointer func_data );

/****************End of Standard button widget *******************************/

/********************* Button on table widget ********************************/

GtkWidget *button_create_on_table(char *label, GtkSignalFunc func, gpointer func_data,
			 GtkWidget *table, int start_col, int end_col, int start_row, int end_row );

/****************End of button on table widget ******************************/

/********************* Button in container widget ***************************/

GtkWidget *button_create_in_container(char *label, GtkSignalFunc func, gpointer func_data,
			GtkWidget *container);

/*************End of Button in container widget *****************************/

/********************* Buttonbox on table widget ****************************/

GtkWidget *button_box_create_on_table(int orientation, int layout,  int border,
										 int spacing, int child_size_x, int child_size_y, GtkWidget *table,
										 int start_col, int end_col, int start_row, int end_row );
GtkWidget *default_button_box_create_on_table(	int orientation, GtkWidget *table,
										 		int start_col, int end_col,
												int start_row, int end_row );

/***************End of Buttonbox on table widget ****************************/

/********************* Label on table widget ********************************/

GtkWidget *label_create_on_table(char *label_text,GtkWidget *table,
				int start_col, int end_col, int start_row, int end_row );

/***************End of Label on table widget ********************************/

/********************* Standard table widget ********************************/

GtkWidget *table_create(int cols, int rows , GtkWidget* window , int border, int homogeneus);

/********************* Standard table widget ********************************/

/********************* Standard window widget ********************************/

GtkWidget *window_create( int x, int y, int border, const char *title, GtkSignalFunc func,
			void *func_data, int modal);

/********************* Standard window widget ********************************/

/********************* Frame on table widget ********************************/

GtkWidget *frame_create_on_table(char *label, GtkWidget *table,
											 int start_col,int end_col, int start_row, int end_row );

/**************End of Frame on table widget ********************************/

/********************* Dialog window widget **********************************/
typedef	struct	Dialog_data			Dialog_data;
struct Dialog_data {
	GtkWidget	*window;
	int *ptr_dialog_retval;
};

void dialog_ok_callback(GtkWidget *widget,  Dialog_data* data);
void dialog_no_callback(GtkWidget *widget,  Dialog_data* data);
void dialog_third_callback(GtkWidget *widget,  Dialog_data *data);
void dialog_check_callback(GtkWidget *widget, int *ptr_check_retval );
int  dialog_window(const char *title, int type, char *third_button_label, int def_button, const char *fmt, ...);
void about_callback(void);

/**************End of Dialog window widget **********************************/

/********************* For the file selection widget *************************/

void file_sel_ok(GtkWidget *widget, int *ptr_ok);
char *file_selector(char *title, int fileop);

/*************End of the file selection widget********************************/

/************** Create a separator on table   ********************************/

GtkWidget *separator_create_on_table(int direction, GtkWidget *table,
									 int start_col,int end_col, int start_row, int end_row );

/**********End of  Create a separator on table********************************/

/************** Create a Multi column scrolled list ************************/

GtkWidget *string_list_create_on_table(int num,GtkSignalFunc func,
										GtkWidget *table, int start_col, int end_col,
										int start_row, int end_row ,int show_hide, ...);
void add_data_to_list(GtkWidget * tree, char *data_string[], int n_columns);

/******** End of Create a Multi column scrolled list ************************/

/************** Create a statusbar **********************************/

GtkWidget * statusbar_create_on_table(GtkWidget *table, GtkWidget **label, char* labeltext,
								int start_col, int end_col, int start_row, int end_row );

/******* End of Create a statusbar **********************************/

/************** Create a Permissions window **********************************/
typedef	struct	Perm_data			Perm_data;
struct Perm_data {
	GtkWidget	*window;
	int *ptr_dialog_retval;
	GtkWidget	*user_entry;
	GtkWidget	*group_entry;
	char* user;
	char* group;
	long uid;
	long gid;
};

int (*chown_func)(const char *, uid_t, gid_t);
int chmodAction(const char *fileName, struct stat *statbuf, void* junk);
int chownAction(const char *fileName, struct stat *statbuf, void* junk);
mode_t get_mode (const char *string);
void set_r_mode_callback(GtkToggleButton *widget, char *position);
void set_w_mode_callback(GtkToggleButton *widget, char *position);
void set_x_mode_callback(GtkToggleButton *widget, char *position);
void set_s_mode_callback(GtkToggleButton *widget, char *position);
void set_t_mode_callback(GtkToggleButton *widget, char *position);
void recursive_callback(GtkToggleButton *widget, int *ptr_recursive);
void perm_quit_callback(GtkWidget *widget,  Perm_data* d);
int PermissionWindow(char *filename);

/*****************End of Permissions window **********************************/

/************** Text window Common  functions ********************************/
typedef	struct	TextBuf			TextBuf;
struct TextBuf {
	GtkWidget	*window;
	GtkWidget	*table;
	GtkWidget	*text;
	int text_modified;
	char *path;
	struct stat sourcefileinfo;
};

FILE *open_file(char *filename );
#ifdef USE_GTK_2_0
int read_and_convert(char * temp_ptr, GtkTextBuffer *buffer, GtkTextIter iter);
#endif
void read_from_fd_and_display_text(GtkWidget *text, FILE *fd, unsigned long filesize, gdouble *progress);
#ifdef USE_GTK_2_4
void toggle_wrap_text(GtkAction *menu_action, TextBuf *tdata);
#else
void toggle_wrap_text(TextBuf *tdata, guint callback_action,  GtkWidget *widget);
#endif

/************** Text window Common  functions ********************************/

/********************* Text file viewer Widget *******************************/

GtkWidget *textpad_create_on_table(	int editable, GtkSignalFunc func, void *func_arg,
									int wrap, int scroll, GtkWidget *table,
									int start_col,int end_col, int start_row, int end_row );
#ifdef USE_GTK_2_4
void textview_quit_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void textview_quit_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
void textview_window_create(char * title, int x, int y, int border, char *filename, FILE *fd);

/**************End of Text file viewer Widget ********************************/

/************** Create a Text Edit window **********************************/

void quit_common(TextBuf *tdata);
void save_common( TextBuf *tdata);
void save_file(char *filename, GtkWidget *text);
void clear_the_buffer(GtkWidget *text, int *text_modified);
int  overwrite(char *msg_str);
#ifdef USE_GTK_2_4
void save_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void save_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
#ifdef USE_GTK_2_4
void save_as_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void save_as_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
#ifdef USE_GTK_2_4
void open_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void open_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
#ifdef USE_GTK_2_4
void new_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void new_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
#ifdef USE_GTK_2_4
void text_quit_callback(GtkAction *menu_action, TextBuf *tdata);
#else
void text_quit_callback(TextBuf *tdata, guint callback_action, GtkWidget *widget);
#endif
void text_window_quit_callback(GtkWidget *widget, gpointer data);
void open_new_common(TextBuf *tdata);
void open_common(TextBuf *tdata);
void text_changed_callback(GtkWidget *widget, int *ptr_text_modified);
void textedit_window_create(char * title, int x, int y, int border, char *filename);

/************** End of a Text Edit window **********************************/

/************ Create a Main Menubar   **************************************/
#ifdef USE_GTK_2_4
typedef	struct	MenuEntry			MenuEntry;
struct MenuEntry {
	gpointer simple_entries;			/*GtkActionEntry*/
	gpointer toggle_entries;			/*GtkToggleActionEntry*/
	gpointer radio_entries;				/*GtkRadioActionEntry*/ 
	int		 num_entries;
	int		 active_radio_entry;		/* Number starting from 0, none=-1 */
	GCallback radio_entry_callback;		/* Radio entry callback */
};

GtkWidget * main_menu_create(GtkWidget *window, MenuEntry *menu_items,const gchar *info, gpointer data);
#else
GtkWidget * main_menu_create( GtkWidget *window, GtkItemFactoryEntry *menu_items, gpointer data);
#endif
/************ Create a Main Menubar   **************************************/

/************** Create a Text Edit window **********************************/

void license_callback(GtkWidget *widget, int *ptr_status);
int about_window_create(char * title,int x, int y, int border, char *labeltext, FILE *fd, int first_time);

/************** Create a Text Edit window **********************************/

/************** Create a Progressbar **********************************/

typedef struct _ProgressData {
	GtkWidget *window;
	GtkWidget *pbar;
	int timer;
	gboolean activity_mode;
	int use_value;
	gdouble value;
	int alive;
	#ifndef USE_GTK_2_0
	GtkAdjustment *adj;
	#endif
} ProgressData;

void progress_destroy(ProgressData *pdata);
gboolean progress_timeout( gpointer data );
void quit_progress(GtkWidget *widget, gpointer ptr_alive);
void quit_progress2(GtkWidget *widget, gpointer ptr_alive);
ProgressData *progressbar_create(int mode, int use_value);
ProgressData *progressbar_create_on_table(int mode, int use_value, GtkWidget *table,
											int start_col, int end_col, int start_row, int end_row);
ProgressData * progressbar_window(char* title, int mode, int use_value, int modal);

/************** Create a Progressbar **********************************/

/************** Create a File System Info Window **********************/

void df(void);

/************** Create a File System Info Window **********************/

/************** Create a Process Manager Window  **********************/

void ps_window(void);
void kill_pid(GtkWidget *widget);
void refresh_ps(GtkWidget *widget, GtkWidget *pslist);
void ps_main(GtkWidget *pslist);
#ifdef USE_GTK_2_0
void pslist_f(GtkWidget *tree);
#else
void pslist_f(GtkWidget *pslist, int row, int column, GdkEventButton *event, gpointer data );
#endif

/************** Create a Process Manager Window  **********************/

/************** Create a Image Viewer Window  *************************/
#ifdef USE_GTK_2_0
/* Not gtk+1.2.10 compatible */
GdkPixbuf * image_resize(GdkPixbuf *img, gint new_width, gint new_height);
GdkPixbuf * image_fit_to_screen (GdkPixbuf *img);
void ViewImageWindow(char *filename);
#endif
/************** Create a Image Viewer Window  *************************/
#endif /* __HAVE_MYGTK_  */
