/* vi: set sw=4 s=4: */
/*
 * Various utility routines put togheter by Tito <tito-wolit@tiscali.it> .
 *
 * Copyright (C) 2002-2003-2004 by Tito Ragusa <tito-wolit@tiscali.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
#ifndef __HAVE_MYLIB_
#define __HAVE_MYLIB_

#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <assert.h>

/* This is to avoid a compiler warning for `index` with -Wshadow and Gtk-+-1.2 libs */
#ifndef USE_GTK_2_0
#ifdef IS_LINUX
#undef  __USE_BSD
#include <string.h>
#define  __USE_BSD
#endif
#else
#include <string.h>
#endif

#ifndef STDERR_FILENO
# define STDERR_FILENO 2
#endif

/* Macros for min/max.  */
#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif

/* Some useful definitions */
#define FALSE   ((int) 0)
#define TRUE    ((int) 1)
#define SKIP	((int) 2)

/* For debugging */
#ifdef DEBUG
#define DBG_FUNCTION_NAME err_msg("%s()",__FUNCTION__)
#else
#define DBG_FUNCTION_NAME
#endif
#ifdef DEBUG
#define debug(x)		x
#else
#define debug(x)
#endif

void err_msg(const char *fmt, ...);
void msg(const char *fmt, ...);
void exit(int) __attribute__(( __noreturn__ ));
void xalloc_die (void *ptr);
void *xrealloc(void *ptr, size_t size);
void *xcalloc(size_t nmemb, size_t size);
void *xmalloc(size_t size);
void *xfree(void *ptr);
void xopen_die (const char *path) __attribute__(( __noreturn__ ));
FILE *xfopen(const char *path, const char *mode);
FILE *xpopen(const char *path, const char *mode);
char *xgetcwd (char *cwd);
char *last_char_is(const char *s, int c);
char *first_char_is(const char *s, int c);
char *xconcat_path_file(const char *path,char *filename);
char *xconcat_subpath_file(const char *path, char *filename);
int  xputs(const char * string);
int  xstrlen(const char* s);

#define SUBSTITUTE 			((int) 1)
#define NOT_SUBSTITUTE		((int) 0)

char *illegal_chars_in_string(char *string,int substitute);

char *fgetc_buf(FILE *fd);
char *xfgetc_nbuf(FILE *fd, int size);
int  xsnprintf(char *dest, size_t size, const char *fmt, ...);
char *xstrncat(char *dest, const char *src, size_t size);
char *xstrdup(const char *s);

#define RETURN_ON_FAILURE				((int) 0)
#define EXIT_ON_FAILURE					((int) 1)
#define EXIT_ERROR						((int) -1)

int xchdir(char *path, int flag);
int xmkdir(const char *pathname , mode_t mode, int flag);
int xchown(const char *path, uid_t owner, gid_t group, int flag);
int xchmod(const char *path, mode_t mode, int flag);
int xfclose_nostdin(FILE *fd);
int xpclose_nostdin(FILE *fd);
char * xmake_message(const char *fmt, ...);

#define UID		((int) 0)
#define GID		((int) 1)

char * get_uid_or_gid (int flag,long idnum);
int file_exists(char *filename, struct stat *buf);
#endif
