/* vi: set sw=4 ts=4: */
/*
 * Move/Rename routine based on Mini mv implementation for busybox
 * Copyright (C) 2005 by Tito Ragusa <tito-wolit@tiscali.it>
 * Copyright (C) 2000 by Matt Kraai <kraai@alumni.carnegiemellon.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/* Mar 16, 2003      Manuel Novoa III   (mjn3@codepoet.org)
 *
 * Size reduction and improved error checking.
 *
 * This move_rename() function is derived from mv_main() of bb's mv.c and was hacked for
 * mini gtk_file_manager by Tito Ragusa <farmatito@tiscali.it> (C) 2003-2004.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <stdlib.h>

#define OPT_FILEUTILS_FORCE       1
#define OPT_FILEUTILS_INTERACTIVE 2

int move_rename(const char *source, const char *dest , int flags)
{
	struct stat source_stat;
	struct stat dest_stat;
	int dest_exists;
	int source_exists;
	int status = 0;
	const char fmt[] = "cannot overwrite %sdirectory with %sdirectory";

	if(source == NULL || dest == NULL) {
		return 1;
	}
	if ((dest_exists = cp_mv_stat(dest, &dest_stat)) < 0) {
		return 1;
	}
	if (!(dest_exists & 2)) {
		goto DO_MOVE;
	}

	dest = concat_path_file(dest, bb_get_last_path_component((char*)source));

	if ((dest_exists = cp_mv_stat(dest, &dest_stat)) < 0) {
		goto RET_1;
	}
DO_MOVE:
	if (dest_exists && !(flags & OPT_FILEUTILS_FORCE)
		&& ((access(dest, W_OK) < 0 && isatty(0)) ||  (flags & OPT_FILEUTILS_INTERACTIVE))) {
		if(dialog_window(STR_MOVE, TWO_BUTTONS, NULL, 2, STR_OVERWRITE_S, dest)==1) /* 1 == Don't overwrite */
			 goto RET_0;
	}

	if (rename(source, dest) < 0) {
		if (errno != EXDEV) {
			bb_perror_msg("unable to rename `%s'", source);
		} else if ((source_exists = cp_mv_stat(source, &source_stat)) >= 0) {
			if (dest_exists) {
				if (dest_exists == 3) {
					if (source_exists != 3) {
						bb_error_msg(fmt, "", "non-");
						goto RET_1;
					}
				} else {
					if (source_exists == 3) {
						bb_error_msg(fmt, "non-", "");
						goto RET_1;
					}
				}
				if (unlink(dest) < 0) {
					bb_perror_msg("cannot remove `%s'", dest);
						goto RET_1;
				}
			}
			if ((copy_file(source, dest, FILEUTILS_RECUR | FILEUTILS_PRESERVE_STATUS) >= 0)
				&& (remove_file(source, FILEUTILS_RECUR | FILEUTILS_FORCE) >= 0)) {
				goto RET_0;
			}
		}
RET_1:
		status = 1;
	}

RET_0:
	return(status);
}

